const { VueLoaderPlugin } = require('vue-loader');
const VuetifyLoaderPlugin = require('vuetify-loader/lib/plugin');

module.exports = {
  filenameHashing: true,
  productionSourceMap: process.env.VUE_APP_DEBUG_SOURCEMAP === 'TRUE' ? true : false,
  assetsDir: './assets',
  publicPath: process.env.VUE_APP_PUBLIC_PATH,
  configureWebpack: (config) => {
    (config.module = {
      rules: [
        {
          test: /\.vue$/,
          use: 'vue-loader',
        },
        {
          test: /\.js$/,
          use: 'babel-loader',
          exclude: /node_modules/,
          options: {
            outputPath: 'assets/js',
          },
        },
        {
          test: /\.css$/,
          use: ['vue-style-loader', 'css-loader'],
          options: {
            outputPath: 'assets/css',
          },
        },
        {
          test: /\.(png|jpe?g|gif)$/i,
          use: 'file-loader',
          options: {
            filenameHashing: false,
            outputPath: 'assets/img/',
            publicPath: 'assets/img/',
            postTransformPublicPath: (p) => `__webpack_public_path__ + ${p}`,
          },
        },
        {
          test: /\.svg$/,
          use: 'vue-svg-loader',
          options: {
            loader: 'vue-svg-loader'
          },
        },
        {
          test: /\.tsx?$/,
          loader: 'ts-loader',
          exclude: /node_modules/,
          options: {
            appendTsSuffixTo: [/\.vue$/],
          },
        },
        {
          test: /\.s[ac]ss$/i,
          use: [
            // Creates `style` nodes from JS strings
            "style-loader",
            // Translates CSS into CommonJS
            "css-loader",
            // Compiles Sass to CSS
            "sass-loader",
          ],
        },
      ],
    }),
      (config.resolve = {
        extensions: ['.ts', '.js', '.vue', '.json'],
        alias: {
          vue$: 'vue/dist/vue.esm.js',
        },
      }),
      (config.plugins = [new VueLoaderPlugin(), new VuetifyLoaderPlugin({
        /**
         * This function will be called for every tag used in each vue component
         * It should return an array, the first element will be inserted into the
         * components array, the second should be a corresponding import
         *
         * originalTag - the tag as it was originally used in the template
         * kebabTag    - the tag normalized to kebab-case
         * camelTag    - the tag normalized to PascalCase
         * path        - a relative path to the current .vue file
         * component   - a parsed representation of the current component
         */
        match(originalTag, { kebabTag, camelTag, path, component }) {
          if (kebabTag.startsWith('core-')) {
            return [
              camelTag,
              `import ${camelTag} from '@/components/core/${camelTag.substring(4)}.vue'`
            ]
          }
        }
      })])
  },
  chainWebpack: (config) => {
    config.resolve.symlinks(false);
    /*remove hash from imgages*/
    config.module
      .rule('images')
      .test(/\.(png|jpe?g|gif)(\?.*)?$/)
      .use('url-loader')
      .loader('url-loader')
      .options({
        name: 'assets/img/[name].[ext]'
      });
  },
  runtimeCompiler: true,
  devServer: {
    port: 8282,
  },
  configureWebpack: {
    optimization: {
      splitChunks: {
        chunks: 'all',
        maxAsyncRequests: 5,
        maxInitialRequests: Infinity,
        minSize: 20000,
        name: true,
        cacheGroups: {
          vendor: {
            test: /[\\/]node_modules[\\/]/,
            name(module) {
              let packageName = module.context.match(
                /[\\/]node_modules[\\/](.*?)([\\/]|$)/
              );
              if (packageName[1] === '@zeedhi') {
                packageName = module.context.match(
                  /[\\/]node_modules[\\/](.*?)[\\/](.*?)([\\/]|$)/
                );
                return `npm.${packageName[1].replace('@', '')}.${packageName[2]
                  }`;
              } else {
                return `npm.${packageName[1].replace('@', '')}`;
              }
            },
          },
          styles: {
            test: /\.css$/,
            name: 'styles',
            chunks: 'all',
            enforce: true,
          },
        },
      },
    },
  },
};
